# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import utils as zun_utils


def do_availability_zone_list(cs, args):
    """Print a list of availability zones."""
    zones = cs.availability_zones.list()
    zun_utils.list_availability_zones(zones)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from osc_lib.command import command
from osc_lib import utils
from oslo_log import log as logging


def _quota_class_columns(quota_class):
    return quota_class._info.keys()


def _get_client(obj, parsed_args):
    obj.log.debug("take_action(%s)" % parsed_args)
    return obj.app.client_manager.container


class UpdateQuotaClass(command.ShowOne):
    """Update the quotas for a quota class"""

    log = logging.getLogger(__name__ + ".UpdateQuotaClass")

    def get_parser(self, prog_name):
        parser = super(UpdateQuotaClass, self).get_parser(prog_name)
        parser.add_argument(
            '--containers',
            metavar='<containers>',
            help='The number of containers allowed per project')
        parser.add_argument(
            '--memory',
            metavar='<memory>',
            help='The number of megabytes of container RAM '
                 'allowed per project')
        parser.add_argument(
            '--cpu',
            metavar='<cpu>',
            help='The number of container cores or vCPUs '
                 'allowed per project')
        parser.add_argument(
            '--disk',
            metavar='<disk>',
            help='The number of gigabytes of container Disk '
                 'allowed per project')
        parser.add_argument(
            'quota_class_name',
            metavar='<quota_class_name>',
            help='The name of quota class')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        opts = {}
        opts['containers'] = parsed_args.containers
        opts['memory'] = parsed_args.memory
        opts['cpu'] = parsed_args.cpu
        opts['disk'] = parsed_args.disk
        quota_class_name = parsed_args.quota_class_name
        quota_class = client.quota_classes.update(
            quota_class_name, **opts)
        columns = _quota_class_columns(quota_class)
        return columns, utils.get_item_properties(quota_class, columns)


class GetQuotaClass(command.ShowOne):
    """List the quotas for a quota class"""

    log = logging.getLogger(__name__ + '.GetQuotaClass')

    def get_parser(self, prog_name):
        parser = super(GetQuotaClass, self).get_parser(prog_name)
        parser.add_argument(
            'quota_class_name',
            metavar='<quota_class_name>',
            help='The name of quota class')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        quota_class_name = parsed_args.quota_class_name
        quota_class = client.quota_classes.get(quota_class_name)
        columns = _quota_class_columns(quota_class)
        return columns, utils.get_item_properties(quota_class, columns)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools
from testtools import matchers

from zunclient.tests.unit import utils
from zunclient.v1 import images


IMAGE1 = {'uuid': '092e2ed7-af11-4fa7-8ffa-c3ee9d5b451a',
          'image_id': 'b8ff79200466',
          'repo': 'fake-repo1',
          'tag': 'latest',
          'size': '1024',
          }
IMAGE2 = {'uuid': '1996ba70-b074-454b-a8fc-0895ae26c7c6',
          'image_id': '21c16b6787c6',
          'repo': 'fake-repo2',
          'tag': 'latest',
          'size': '1024',
          }
IMAGE3 = {'uuid': '1267gf34-34e4-tf4b-b84c-1345avf6c7c6',
          'image_id': '24r5tt6y87c6',
          'image': 'fake-name3',
          'tag': 'latest',
          'size': '1024',
          'image_driver': 'fake-driver',
          }
SEARCH_IMAGE = {'image': 'fake-name3',
                'image_driver': 'fake-driver',
                }


fake_responses = {
    '/v1/images/':
    {
        'GET': (
            {},
            {'images': [IMAGE1, IMAGE2]},
        ),
    },
    '/v1/images/?limit=2':
    {
        'GET': (
            {},
            {'images': [IMAGE1, IMAGE2]},
        ),
    },
    '/v1/images/?marker=%s' % IMAGE2['image_id']:
    {
        'GET': (
            {},
            {'images': [IMAGE1, IMAGE2]},
        ),
    },
    '/v1/images/?limit=2&marker=%s' % IMAGE2['image_id']:
    {
        'GET': (
            {},
            {'images': [IMAGE2, IMAGE1]},
        ),
    },
    '/v1/images/?sort_dir=asc':
    {
        'GET': (
            {},
            {'images': [IMAGE1, IMAGE2]},
        ),
    },
    '/v1/images/?sort_key=image_id':
    {
        'GET': (
            {},
            {'images': [IMAGE1, IMAGE2]},
        ),
    },
    '/v1/images/?sort_key=image_id&sort_dir=desc':
    {
        'GET': (
            {},
            {'images': [IMAGE2, IMAGE1]},
        ),
    },
    '/v1/images/%s/search?image_driver=%s' % (IMAGE3['image'],
                                              IMAGE3['image_driver']):
    {
        'GET': (
            {},
            {'images': [IMAGE3]},
        ),
    },
}


class ImageManagerTest(testtools.TestCase):

    def setUp(self):
        super(ImageManagerTest, self).setUp()
        self.api = utils.FakeAPI(fake_responses)
        self.mgr = images.ImageManager(self.api)

    def test_image_list(self):
        images = self.mgr.list()
        expect = [
            ('GET', '/v1/images/', {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(images, matchers.HasLength(2))

    def _test_image_list_with_filters(
            self, limit=None, marker=None,
            sort_key=None, sort_dir=None,
            expect=[]):
        images_filter = self.mgr.list(limit=limit, marker=marker,
                                      sort_key=sort_key,
                                      sort_dir=sort_dir)
        self.assertEqual(expect, self.api.calls)
        self.assertThat(images_filter, matchers.HasLength(2))

    def test_image_list_with_limit(self):
        expect = [
            ('GET', '/v1/images/?limit=2', {}, None),
        ]
        self._test_image_list_with_filters(
            limit=2,
            expect=expect)

    def test_image_list_with_marker(self):
        expect = [
            ('GET', '/v1/images/?marker=%s' % IMAGE2['image_id'],
             {}, None),
        ]
        self._test_image_list_with_filters(
            marker=IMAGE2['image_id'],
            expect=expect)

    def test_image_list_with_marker_limit(self):
        expect = [
            ('GET', '/v1/images/?limit=2&marker=%s' % IMAGE2['image_id'],
             {}, None),
        ]
        self._test_image_list_with_filters(
            limit=2, marker=IMAGE2['image_id'],
            expect=expect)

    def test_image_list_with_sort_dir(self):
        expect = [
            ('GET', '/v1/images/?sort_dir=asc',
             {}, None),
        ]
        self._test_image_list_with_filters(
            sort_dir='asc',
            expect=expect)

    def test_image_list_with_sort_key(self):
        expect = [
            ('GET', '/v1/images/?sort_key=image_id',
             {}, None),
        ]
        self._test_image_list_with_filters(
            sort_key='image_id',
            expect=expect)

    def test_image_list_with_sort_key_dir(self):
        expect = [
            ('GET', '/v1/images/?sort_key=image_id&sort_dir=desc',
             {}, None),
        ]
        self._test_image_list_with_filters(
            sort_key='image_id', sort_dir='desc',
            expect=expect)

    def test_image_search(self):
        images = self.mgr.search_image(**SEARCH_IMAGE)
        url = '/v1/images/%s/search?image_driver=%s' \
              % (IMAGE3['image'], IMAGE3['image_driver'])
        expect = [
            ('GET', url, {}, None),
        ]
        self.assertEqual(expect, self.api.calls)
        self.assertThat(images, matchers.HasLength(1))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import yaml

from oslo_serialization import jsonutils

from zunclient.common import cliutils as utils
from zunclient.common import utils as zun_utils


@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='The last host UUID of the previous page; '
                'displays list of hosts after "marker".')
@utils.arg('--limit',
           metavar='<limit>',
           type=int,
           help='Maximum number of hosts to return')
@utils.arg('--sort-key',
           metavar='<sort-key>',
           help='Column to sort results by')
@utils.arg('--sort-dir',
           metavar='<sort-dir>',
           choices=['desc', 'asc'],
           help='Direction to sort. "asc" or "desc".')
def do_host_list(cs, args):
    """Print a list of available host."""
    opts = {}
    opts['marker'] = args.marker
    opts['limit'] = args.limit
    opts['sort_key'] = args.sort_key
    opts['sort_dir'] = args.sort_dir
    opts = zun_utils.remove_null_parms(**opts)
    hosts = cs.hosts.list(**opts)
    columns = ('uuid', 'hostname', 'mem_total', 'cpus', 'disk_total')
    utils.print_list(hosts, columns,
                     {'versions': zun_utils.print_list_field('versions')},
                     sortby_index=None)


@utils.arg('host',
           metavar='<host>',
           help='ID or name of the host to show.')
@utils.arg('-f', '--format',
           metavar='<format>',
           action='store',
           choices=['json', 'yaml', 'table'],
           default='table',
           help='Print representation of the host.'
                'The choices of the output format is json,table,yaml.'
                'Defaults to table.')
def do_host_show(cs, args):
    """Show details of a host."""
    host = cs.hosts.get(args.host)
    if args.format == 'json':
        print(jsonutils.dumps(host._info, indent=4, sort_keys=True))
    elif args.format == 'yaml':
        print(yaml.safe_dump(host._info, default_flow_style=False))
    elif args.format == 'table':
        utils.print_dict(host._info)



# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# W0603: Using the global statement
# W0621: Redefining name %s from outer scope
# pylint: disable=W0603,W0621

import collections
import getpass
import inspect
import os
import sys
import textwrap

import decorator
from oslo_utils import encodeutils
from oslo_utils import strutils
import prettytable

from zunclient.i18n import _


class MissingArgs(Exception):
    """Supplied arguments are not sufficient for calling a function."""
    def __init__(self, missing):
        self.missing = missing
        msg = _("Missing arguments: %s") % ", ".join(missing)
        super(MissingArgs, self).__init__(msg)


def validate_args(fn, *args, **kwargs):
    """Check that the supplied args are sufficient for calling a function.

    >>> validate_args(lambda a: None)
    Traceback (most recent call last):
        ...
    MissingArgs: Missing argument(s): a
    >>> validate_args(lambda a, b, c, d: None, 0, c=1)
    Traceback (most recent call last):
        ...
    MissingArgs: Missing argument(s): b, d

    :param fn: the function to check
    :param arg: the positional arguments supplied
    :param kwargs: the keyword arguments supplied
    """
    argspec = inspect.getargspec(fn)

    num_defaults = len(argspec.defaults or [])
    required_args = argspec.args[:len(argspec.args) - num_defaults]

    def isbound(method):
        return getattr(method, '__self__', None) is not None

    if isbound(fn):
        required_args.pop(0)

    missing = [arg for arg in required_args if arg not in kwargs]
    missing = missing[len(args):]
    if missing:
        raise MissingArgs(missing)


def arg(*args, **kwargs):
    """Decorator for CLI args.

    Example:

    >>> @arg("name", help="Name of the new entity")
    ... def entity_create(args):
    ...     pass
    """
    def _decorator(func):
        add_arg(func, *args, **kwargs)
        return func
    return _decorator


def exclusive_arg(group_name, *args, **kwargs):
    """Decorator for CLI mutually exclusive args."""
    def _decorator(func):
        required = kwargs.pop('required', None)
        add_exclusive_arg(func, group_name, required, *args, **kwargs)
        return func
    return _decorator


def env(*args, **kwargs):
    """Returns the first environment variable set.

    If all are empty, defaults to '' or keyword arg `default`.
    """
    for arg in args:
        value = os.environ.get(arg)
        if value:
            return value
    return kwargs.get('default', '')


def add_arg(func, *args, **kwargs):
    """Bind CLI arguments to a shell.py `do_foo` function."""

    if not hasattr(func, 'arguments'):
        func.arguments = []

    # NOTE(sirp): avoid dups that can occur when the module is shared across
    # tests.
    if (args, kwargs) not in func.arguments:
        # Because of the semantics of decorator composition if we just append
        # to the options list positional options will appear to be backwards.
        func.arguments.insert(0, (args, kwargs))


def add_exclusive_arg(func, group_name, required, *args, **kwargs):
    """Bind CLI mutally exclusive arguments to a shell.py `do_foo` function."""

    if not hasattr(func, 'exclusive_args'):
        func.exclusive_args = collections.defaultdict(list)
        # Default required to False
        func.exclusive_args['__required__'] = collections.defaultdict(bool)

    # NOTE(sirp): avoid dups that can occur when the module is shared across
    # tests.
    if (args, kwargs) not in func.exclusive_args[group_name]:
        # Because of the semantics of decorator composition if we just append
        # to the options list positional options will appear to be backwards.
        func.exclusive_args[group_name].insert(0, (args, kwargs))
        if required is not None:
            func.exclusive_args['__required__'][group_name] = required


def unauthenticated(func):
    """Adds 'unauthenticated' attribute to decorated function.

    Usage:

    >>> @unauthenticated
    ... def mymethod(f):
    ...     pass
    """
    func.unauthenticated = True
    return func


def isunauthenticated(func):
    """Checks if the function does not require authentication.

    Mark such functions with the `@unauthenticated` decorator.

    :returns: bool
    """
    return getattr(func, 'unauthenticated', False)


def print_list(objs, fields, formatters=None, sortby_index=0,
               mixed_case_fields=None, field_labels=None):
    """Print a list or objects as a table, one row per object.

    :param objs: iterable of :class:`Resource`
    :param fields: attributes that correspond to columns, in order
    :param formatters: `dict` of callables for field formatting
    :param sortby_index: index of the field for sorting table rows
    :param mixed_case_fields: fields corresponding to object attributes that
        have mixed case names (e.g., 'serverId')
    :param field_labels: Labels to use in the heading of the table, default to
        fields.
    """
    formatters = formatters or {}
    mixed_case_fields = mixed_case_fields or []
    field_labels = field_labels or fields
    if len(field_labels) != len(fields):
        raise ValueError(_("Field labels list %(labels)s has different number "
                           "of elements than fields list %(fields)s"),
                         {'labels': field_labels, 'fields': fields})

    if sortby_index is None:
        kwargs = {}
    else:
        kwargs = {'sortby': field_labels[sortby_index]}
    pt = prettytable.PrettyTable(field_labels)
    pt.align = 'l'

    for o in objs:
        row = []
        for field in fields:
            if field in formatters:
                row.append(formatters[field](o))
            else:
                if field in mixed_case_fields:
                    field_name = field.replace(' ', '_')
                else:
                    field_name = field.lower().replace(' ', '_')
                data = getattr(o, field_name, '')
                row.append(data)
        pt.add_row(row)

    print(encodeutils.safe_encode(pt.get_string(**kwargs)).decode())


def keys_and_vals_to_strs(dictionary):
    """Recursively convert a dictionary's keys and values to strings.

    :param dictionary: dictionary whose keys/vals are to be converted to strs
    """
    def to_str(k_or_v):
        if isinstance(k_or_v, dict):
            return keys_and_vals_to_strs(k_or_v)
        elif isinstance(k_or_v, str):
            return str(k_or_v)
        else:
            return k_or_v
    return dict((to_str(k), to_str(v)) for k, v in dictionary.items())


def print_dict(dct, dict_property="Property", wrap=0, value_fields=None):
    """Print a `dict` as a table of two columns.

    :param dct: `dict` to print
    :param dict_property: name of the first column
    :param wrap: wrapping for the second column
    :param value_fields: attributes that correspond to columns, in order
    """
    pt = prettytable.PrettyTable([dict_property, 'Value'])
    if value_fields:
        pt = prettytable.PrettyTable([dict_property] + list(value_fields))
    pt.align = 'l'
    for k, v in dct.items():
        # convert dict to str to check length
        if isinstance(v, dict) and not value_fields:
            v = str(keys_and_vals_to_strs(v))
        if wrap > 0:
            v = textwrap.fill(str(v), wrap)
        elif wrap < 0:
            raise ValueError(_("Wrap argument should be a positive integer"))
        # if value has a newline, add in multiple rows
        # e.g. fault with stacktrace
        if v and isinstance(v, str) and r'\n' in v:
            lines = v.strip().split(r'\n')
            col1 = k
            for line in lines:
                pt.add_row([col1, line])
                col1 = ''
        elif isinstance(v, dict):
            vals = [v[field] for field in v if field in value_fields]
            pt.add_row([k] + vals)
        elif isinstance(v, list):
            val = str([str(i) for i in v])
            pt.add_row([k, val])
        else:
            pt.add_row([k, v])

    print(encodeutils.safe_encode(pt.get_string()).decode())


def get_password(max_password_prompts=3):
    """Read password from TTY."""
    verify = strutils.bool_from_string(env("OS_VERIFY_PASSWORD"))
    pw = None
    if hasattr(sys.stdin, "isatty") and sys.stdin.isatty():
        # Check for Ctrl-D
        try:
            for __ in range(max_password_prompts):
                pw1 = getpass.getpass("OS Password: ")
                if verify:
                    pw2 = getpass.getpass("Please verify: ")
                else:
                    pw2 = pw1
                if pw1 == pw2 and pw1:
                    pw = pw1
                    break
        except EOFError:
            pass
    return pw


def service_type(stype):
    """Adds 'service_type' attribute to decorated function.

    Usage:

    .. code-block:: python

       @service_type('volume')
       def mymethod(f):
       ...
    """
    def inner(f):
        f.service_type = stype
        return f
    return inner


def get_service_type(f):
    """Retrieves service type from function."""
    return getattr(f, 'service_type', None)


def pretty_choice_list(cl):
    return ', '.join("'%s'" % i for i in cl)


def exit(msg=''):
    if msg:
        print(msg, file=sys.stderr)
    sys.exit(1)


def deprecated(message):
    @decorator.decorator
    def wrapper(func, *args, **kwargs):
        print(message)
        return func(*args, **kwargs)
    return wrapper



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import configparser as config_parser
import os

from tempest.lib.cli import base

DEFAULT_CONFIG_FILE = os.path.join(os.path.dirname(__file__), 'test.conf')


class FunctionalTestBase(base.ClientTestBase):
    """Container base class, calls to zunclient."""

    def setUp(self):
        super(FunctionalTestBase, self).setUp()
        self.client = self._get_clients()

    def _get_clients(self):
        # NOTE(aarefiev): {toxinidir} is a current working directory, so
        # the tox env path is {toxinidir}/.tox
        cli_dir = os.path.join(os.path.abspath('.'), '.tox/functional/bin')

        config = self._get_config()
        if config.get('os_auth_url'):
            client = base.CLIClient(cli_dir=cli_dir,
                                    username=config['os_username'],
                                    password=config['os_password'],
                                    tenant_name=config['os_project_name'],
                                    uri=config['os_auth_url'])
            for keystone_object in 'user', 'project':
                domain_attr = 'os_%s_domain_id' % keystone_object
                if config.get(domain_attr):
                    setattr(self, domain_attr, config[domain_attr])
        else:
            self.zun_url = config['zun_url']
            self.os_auth_token = config['os_auth_token']
            client = base.CLIClient(cli_dir=cli_dir,
                                    zun_url=self.zun_url,
                                    os_auth_token=self.os_auth_token)
        return client

    def _get_config(self):
        config_file = os.environ.get('ZUNCLIENT_TEST_CONFIG',
                                     DEFAULT_CONFIG_FILE)
        config = config_parser.ConfigParser()
        if not config.read(config_file):
            self.skipTest('Skipping, no test config found @ %s' % config_file)
        try:
            auth_strategy = config.get('functional', 'auth_strategy')
        except config_parser.NoOptionError:
            auth_strategy = 'keystone'
        if auth_strategy not in ['keystone', 'noauth']:
            raise self.fail(
                'Invalid auth type specified: %s in functional must be '
                'one of: [keystone, noauth]' % auth_strategy)

        conf_settings = []
        keystone_v3_conf_settings = []
        if auth_strategy == 'keystone':
            conf_settings += ['os_auth_url', 'os_username',
                              'os_password', 'os_project_name',
                              'os_identity_api_version']
            keystone_v3_conf_settings += ['os_user_domain_id',
                                          'os_project_domain_id']
        else:
            conf_settings += ['os_auth_token', 'zun_url']

        cli_flags = {}
        missing = []
        for c in conf_settings + keystone_v3_conf_settings:
            try:
                cli_flags[c] = config.get('functional', c)
            except config_parser.NoOptionError:
                # NOTE(vdrok): Here we ignore the absence of KS v3 options as
                # v2 may be used. Keystone client will do the actual check of
                # the parameters' correctness.
                if c not in keystone_v3_conf_settings:
                    missing.append(c)
        if missing:
            self.fail('Missing required setting in test.conf (%(conf)s) for '
                      'auth_strategy=%(auth)s: %(missing)s' %
                      {'conf': config_file,
                       'auth': auth_strategy,
                       'missing': ','.join(missing)})
        return cli_flags

    def _cmd_no_auth(self, cmd, action, flags='', params=''):
        """Execute given command with noauth attributes.

        :param cmd: command to be executed
        :type cmd: string
        :param action: command on cli to run
        :type action: string
        :param flags: optional cli flags to use
        :type flags: string
        :param params: optional positional args to use
        :type params: string
        """
        flags = ('--os_auth_token %(token)s --zun_url %(url)s %(flags)s'
                 %
                 {'token': self.os_auth_token,
                  'url': self.zun_url,
                  'flags': flags})
        return base.execute(cmd, action, flags, params,
                            cli_dir=self.client.cli_dir)

    def _zun(self, action, cmd='zun', flags='', params='', merge_stderr=False):
        """Execute Zun command for the given action.

        :param action: the cli command to run using Zun
        :type action: string
        :param cmd: the base of cli command to run
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param merge_stderr: whether to merge stderr into the result
        :type merge_stderr: bool
        """
        if cmd == 'openstack':
            config = self._get_config()
            id_api_version = config['os_identity_api_version']
            flags += ' --os-identity-api-version {0}'.format(id_api_version)
        else:
            flags += ' --os-endpoint-type publicURL'

        if hasattr(self, 'os_auth_token'):
            return self._cmd_no_auth(cmd, action, flags, params)
        else:
            for keystone_object in 'user', 'project':
                domain_attr = 'os_%s_domain_id' % keystone_object
                if hasattr(self, domain_attr):
                    flags += ' --os-%(ks_obj)s-domain-id %(value)s' % {
                        'ks_obj': keystone_object,
                        'value': getattr(self, domain_attr)
                    }
            return self.client.cmd_with_auth(
                cmd, action, flags, params, merge_stderr=merge_stderr)

    def zun(self, action, flags='', params='', parse=True):
        """Return parsed list of dicts with basic item info.

        :param action: the cli command to run using Container
        :type action: string
        :param flags: any optional cli flags to use
        :type flags: string
        :param params: any optional positional args to use
        :type params: string
        :param parse: return parsed list or raw output
        :type parse: bool
        """
        output = self._zun(action=action, flags=flags, params=params)
        return self.parser.listing(output) if parse else output

    def get_table_headers(self, action, flags='', params=''):
        output = self._zun(action=action, flags=flags, params=params)
        table = self.parser.table(output)
        return table['headers']

    def assertTableHeaders(self, field_names, table_headers):
        """Assert that field_names and table_headers are equal.

        :param field_names: field names from the output table of the cmd
        :param table_headers: table headers output from cmd
        """
        self.assertEqual(sorted(field_names), sorted(table_headers))

    def list_containers(self, params=''):
        return self.zun('container-list', params=params)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import base


class QuotaClass(base.Resource):
    def __repr__(self):
        return "<QuotaClass %s>" % self._info


class QuotaClassManager(base.Manager):
    resource_class = QuotaClass

    @staticmethod
    def _path(quota_class_name):
        return '/v1/quota_classes/{}' . format(quota_class_name)

    def get(self, quota_class_name):
        return self._list(self._path(quota_class_name))[0]

    def update(self, quota_class_name, containers=None,
               memory=None, cpu=None, disk=None):
        resources = {}
        if cpu is not None:
            resources['cpu'] = cpu
        if memory is not None:
            resources['memory'] = memory
        if containers is not None:
            resources['containers'] = containers
        if disk is not None:
            resources['disk'] = disk
        return self._update(self._path(quota_class_name),
                            resources, method='PUT')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import base


class Action(base.Resource):
    def __repr__(self):
        return "<Action %s>" % self._info


class ActionManager(base.Manager):
    resource_class = Action

    @staticmethod
    def _path(container, request_id=None):

        if request_id:
            return '/v1/containers/%s/container_actions/%s' % (container,
                                                               request_id)
        else:
            return '/v1/containers/%s/container_actions' % container

    def list(self, container):
        """Retrieve a list of actions.

        :returns: A list of actions.

        """

        return self._list(self._path(container), "containerActions")

    def get(self, container, request_id):
        try:
            return self._list(self._path(container, request_id))[0]
        except IndexError:
            return None



#    Copyright (c) 2016 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import time

from zunclient.tests.functional.osc.v1 import base


@ddt.ddt
class ContainerTests(base.TestCase):
    """Functional tests for container commands."""

    def setUp(self):
        super(ContainerTests, self).setUp()

    def test_list(self):
        """Check container list command.

        """
        container = self.container_create(name='test_list')
        container_list = self.container_list()
        self.assertIn(container['name'], [x['name'] for x in
                      container_list])
        self.assertIn(container['uuid'], [x['uuid'] for x in
                      container_list])

        # Now delete the container and then see the list
        self.container_delete(container['name'])

    def test_create(self):
        """Check container create command.

        """
        container_info = self.container_create(name='test_create')
        self.assertEqual(container_info['name'], 'test_create')
        self.assertEqual(container_info['image'], 'cirros')
        container_list = self.container_list()
        self.assertIn('test_create', [x['name'] for x in container_list])
        self.container_delete(container_info['name'])

    def test_delete(self):
        """Check container delete command with name/UUID argument.

        Test steps:
        1) Create container in setUp.
        2) Delete container by name/UUID.
        3) Check that container deleted successfully.
        """
        container = self.container_create(name='test_delete')
        container_list = self.container_list()
        self.assertIn(container['name'],
                      [x['name'] for x in container_list])
        self.assertIn(container['uuid'],
                      [x['uuid'] for x in container_list])
        count = 0
        while count < 5:
            container = self.container_show(container['name'])
            if container['status'] == 'Created':
                break
            if container['status'] == 'Error':
                break
            time.sleep(2)
            count = count + 1
        self.container_delete(container['name'])
        # Wait for the container to be deleted
        count = 0
        while count < 5:
            container_list = self.container_list()
            if container['name'] not in [x['name'] for x in container_list]:
                break
            time.sleep(2)
            count = count + 1
        container_list = self.container_list()
        self.assertNotIn(container['name'],
                         [x['name'] for x in container_list])
        self.assertNotIn(container['uuid'],
                         [x['uuid'] for x in container_list])

    def test_show(self):
        """Check container show command with name and UUID arguments.

        Test steps:
        1) Create container in setUp.
        2) Show container calling it with name and UUID arguments.
        3) Check name, uuid and image in container show output.
        """
        container = self.container_create(name='test_show')
        self.container_show(container['name'])
        self.assertEqual(container['name'], container['name'])
        self.assertEqual(container['image'], container['image'])
        self.container_delete(container['name'])

    def test_execute(self):
        """Check container execute command with name and UUID arguments.

        Test steps:
        1) Create container in setUp.
        2) Execute command calling it with name and UUID arguments.
        3) Check the container logs.
        """
        container = self.container_run(name='test_execute')
        count = 0
        while count < 50:
            container = self.container_show(container['name'])
            if container['status'] == 'Running':
                break
            if container['status'] == 'Error':
                break
            time.sleep(2)
            count = count + 1
        command = "sh -c 'echo hello'"
        result = self.container_execute(container['name'], command)
        self.assertIn('hello', result)
        self.container_delete(container['name'])



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse

from zunclient import api_versions
from zunclient.common import base
from zunclient.common import utils
from zunclient import exceptions


CREATION_ATTRIBUTES = ['name', 'image', 'command', 'cpu', 'memory',
                       'environment', 'workdir', 'labels', 'image_pull_policy',
                       'restart_policy', 'interactive', 'image_driver',
                       'security_groups', 'hints', 'nets', 'auto_remove',
                       'runtime', 'hostname', 'mounts', 'disk',
                       'availability_zone', 'auto_heal', 'privileged',
                       'exposed_ports', 'healthcheck', 'registry', 'tty',
                       'host', 'entrypoint']


class Container(base.Resource):
    def __repr__(self):
        return "<Container %s>" % self._info


class ContainerManager(base.Manager):
    resource_class = Container

    @staticmethod
    def _path(id=None):

        if id:
            return '/v1/containers/%s' % id
        else:
            return '/v1/containers'

    def list(self, marker=None, limit=None, sort_key=None,
             sort_dir=None, all_projects=False, **kwargs):
        """Retrieve a list of containers.

        :param all_projects: Optional, list containers in all projects

        :param marker: Optional, the UUID of a containers, eg the last
                       containers from a previous result set. Return
                       the next result set.
        :param limit: The maximum number of results to return per
                      request, if:

            1) limit > 0, the maximum number of containers to return.
            2) limit param is NOT specified (None), the number of items
               returned respect the maximum imposed by the ZUN API
               (see Zun's api.max_limit option).

        :param sort_key: Optional, field used for sorting.

        :param sort_dir: Optional, direction of sorting, either 'asc' (the
                         default) or 'desc'.

        :returns: A list of containers.

        """
        if limit is not None:
            limit = int(limit)

        filters = utils.common_filters(marker, limit, sort_key,
                                       sort_dir, all_projects)
        path = ''
        if filters:
            path += '?' + '&'.join(filters)

        if limit is None:
            return self._list(self._path(path),
                              "containers", qparams=kwargs)
        else:
            return self._list_pagination(self._path(path),
                                         "containers",
                                         limit=limit)

    def get(self, id, **kwargs):
        try:
            return self._list(self._path(id),
                              qparams=kwargs)[0]
        except IndexError:
            return None

    def create(self, **kwargs):
        self._process_command(kwargs)
        self._process_mounts(kwargs)
        self._process_tty(kwargs)

        new = {}
        for (key, value) in kwargs.items():
            if key in CREATION_ATTRIBUTES:
                new[key] = value
            else:
                raise exceptions.InvalidAttribute(
                    "Key must be in %s" % ','.join(CREATION_ATTRIBUTES))
        return self._create(self._path(), new)

    def _process_command(self, kwargs):
        cmd_microversion = api_versions.APIVersion("1.20")
        if self.api_version < cmd_microversion:
            command = kwargs.pop('command', None)
            if command:
                kwargs['command'] = utils.parse_command(command)

    def _process_mounts(self, kwargs):
        mounts = kwargs.get('mounts', None)
        if mounts:
            for mount in mounts:
                if mount.get('type') == 'bind':
                    mount['source'] = utils.encode_file_data(mount['source'])

    def _process_tty(self, kwargs):
        tty_microversion = api_versions.APIVersion("1.36")
        if self.api_version >= tty_microversion:
            if 'interactive' in kwargs and 'tty' not in kwargs:
                kwargs['tty'] = kwargs['interactive']

    def delete(self, id, **kwargs):
        return self._delete(self._path(id),
                            qparams=kwargs)

    def _action(self, id, action, method='POST', qparams=None, **kwargs):
        if qparams:
            action = "%s?%s" % (action,
                                parse.urlencode(qparams))
        kwargs.setdefault('headers', {})
        kwargs['headers'].setdefault('Content-Length', '0')
        resp, body = self.api.json_request(method,
                                           self._path(id) + action,
                                           **kwargs)
        return resp, body

    def start(self, id):
        return self._action(id, '/start')

    def stop(self, id, timeout):
        return self._action(id, '/stop',
                            qparams={'timeout': timeout})

    def rebuild(self, id, **kwargs):
        return self._action(id, '/rebuild',
                            qparams=kwargs)

    def restart(self, id, timeout):
        return self._action(id, '/reboot',
                            qparams={'timeout': timeout})

    def pause(self, id):
        return self._action(id, '/pause')

    def unpause(self, id):
        return self._action(id, '/unpause')

    def logs(self, id, **kwargs):
        if kwargs['stdout'] is False and kwargs['stderr'] is False:
            kwargs['stdout'] = True
            kwargs['stderr'] = True
        return self._action(id, '/logs', method='GET',
                            qparams=kwargs)[1]

    def execute(self, id, **kwargs):
        return self._action(id, '/execute',
                            qparams=kwargs)[1]

    def execute_resize(self, id, exec_id, width, height):
        self._action(id, '/execute_resize',
                     qparams={'exec_id': exec_id, 'w': width, 'h': height})[1]

    def kill(self, id, signal=None):
        return self._action(id, '/kill',
                            qparams={'signal': signal})[1]

    def run(self, **kwargs):
        self._process_command(kwargs)
        self._process_mounts(kwargs)
        self._process_tty(kwargs)

        if not set(kwargs).issubset(CREATION_ATTRIBUTES):
            raise exceptions.InvalidAttribute(
                "Key must be in %s" % ','.join(CREATION_ATTRIBUTES))
        else:
            return self._create(self._path() + '?run=true', kwargs)

    def rename(self, id, name):
        return self._action(id, '/rename',
                            qparams={'name': name})

    def update(self, id, **patch):
        return self._update(self._path(id), patch)

    def attach(self, id):
        return self._action(id, '/attach', method='GET')[1]

    def resize(self, id, width, height):
        return self._action(id, '/resize',
                            qparams={'w': width, 'h': height})[1]

    def top(self, id, ps_args=None):
        return self._action(id, '/top', method='GET',
                            qparams={'ps_args': ps_args})[1]

    def get_archive(self, id, path):
        res = self._action(id, '/get_archive', method='GET',
                           qparams={'path': path})[1]
        # API version 1.25 or later will return Base64-encoded data
        if self.api_version >= api_versions.APIVersion("1.25"):
            res['data'] = utils.decode_file_data(res['data'])
        else:
            res['data'] = res['data'].encode()
        return res

    def put_archive(self, id, path, data):
        # API version 1.25 or later will expect Base64-encoded data
        if self.api_version >= api_versions.APIVersion("1.25"):
            data = utils.encode_file_data(data)
        return self._action(id, '/put_archive',
                            qparams={'path': path},
                            body={'data': data})

    def stats(self, id):
        return self._action(id, '/stats', method='GET')[1]

    def commit(self, id, repository, tag=None):
        if tag is not None:
            return self._action(id, '/commit', qparams={
                                'repository': repository, 'tag': tag})[1]
        else:
            return self._action(id, '/commit', qparams={
                                'repository': repository})[1]

    def add_security_group(self, id, security_group):
        return self._action(id, '/add_security_group',
                            qparams={'name': security_group})

    def network_detach(self, container, **kwargs):
        return self._action(container, '/network_detach',
                            qparams=kwargs)

    def network_attach(self, container, **kwargs):
        return self._action(container, '/network_attach',
                            qparams=kwargs)

    def network_list(self, container):
        return self._list(self._path(container) + '/network_list',
                          "networks")

    def remove_security_group(self, id, security_group):
        return self._action(id, '/remove_security_group',
                            qparams={'name': security_group})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils

from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from zunclient.tests.functional import base


class TestCase(base.FunctionalTestBase):

    def openstack(self, *args, **kwargs):
        return self._zun(cmd='openstack', *args, **kwargs)

    def get_opts(self, fields=None, output_format='json'):
        """Get options for OSC output fields format.

        :param List fields: List of fields to get
        :param String output_format: Select output format
        :return: String of formatted options
        """
        if not fields:
            return ' -f {0}'.format(output_format)
        return ' -f {0} {1}'.format(output_format,
                                    ' '.join(['-c ' + it for it in fields]))

    def container_create(self, image='cirros', name=None, params=''):
        """Create container and add cleanup.

        :param String image: Image for a new container
        :param String name: Name for a new container
        :param String params: Additional args and kwargs
        :return: JSON object of created container
        """
        if not name:
            name = data_utils.rand_name('container')

        opts = self.get_opts()
        output = self.openstack('appcontainer create {0}'
                                ' --name {1} {2} {3}'
                                .format(opts, name, image, params))
        container = jsonutils.loads(output)

        if not output:
            self.fail('Container has not been created!')
        return container

    def container_run(self, image='cirros', name=None, params='sleep 100000'):
        """Run container and add cleanup.

        :param String image: Image for a new container
        :param String name: Name for a new container
        :param String params: Additional args and kwargs
        :return: JSON object of created container
        """
        if not name:
            name = data_utils.rand_name('container')

        opts = self.get_opts()
        output = self.openstack('appcontainer run {0}'
                                ' --name {1} {2} {3}'
                                .format(opts, name, image, params))
        container = jsonutils.loads(output)

        if not output:
            self.fail('Container has not run!')
        return container

    def container_delete(self, identifier, force=True,
                         ignore_exceptions=False):
        """Try to delete container by name or UUID.

        :param String identifier: Name or UUID of the container
        :param Bool ignore_exceptions: Ignore exception (needed for cleanUp)
        :return: raw values output
        :raise: CommandFailed exception when command fails
                to delete a container
        """
        arg = '--force' if force else ''
        try:
            return self.openstack('appcontainer delete {0} {1}'
                                  .format(arg, identifier))
        except exceptions.CommandFailed:
            if not ignore_exceptions:
                raise

    def container_list(self, fields=None, params=''):
        """List Container.

        :param List fields: List of fields to show
        :param String params: Additional kwargs
        :return: list of JSON container objects
        """
        opts = self.get_opts(fields=fields)
        output = self.openstack('appcontainer list {0} {1}'
                                .format(opts, params))
        return jsonutils.loads(output)

    def container_show(self, identifier, fields=None, params=''):
        """Show specified container.

        :param String identifier: Name or UUID of the container
        :param List fields: List of fields to show
        :param List params: Additional kwargs
        :return: JSON object of container
        """
        opts = self.get_opts(fields)
        output = self.openstack('appcontainer show {0} {1} {2}'
                                .format(opts, identifier, params))
        return jsonutils.loads(output)

    def container_rename(self, identifier, name):
        """Rename specified container.

        :param String identifier: Name or UUID of the container
        :param String name: new name for the container
        """
        self.openstack('appcontainer rename {0} {1}'
                       .format(identifier, name))

    def container_execute(self, identifier, command):
        """Execute in specified container.

        :param String identifier: Name or UUID of the container
        :param String command: command execute in the container
        """
        return self.openstack('appcontainer exec {0} {1}'
                              .format(identifier, command))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import urllib

from zunclient.common import base


class Version(base.Resource):
    def __repr__(self):
        return "<Version>"


class VersionManager(base.Manager):
    resource_class = Version

    def list(self):
        endpoint = self.api.get_endpoint()
        url = urllib.parse.urlparse(endpoint)
        # NOTE(hongbin): endpoint URL has at least 2 formats:
        #   1. the classic (legacy) endpoint:
        #       http://{host}:{optional_port}/v1/
        #   2. under wsgi:
        #       http://{host}:{optional_port}/container/v1
        if url.path.endswith("v1") or "/v1/" in url.path:
            # this way should handle all 2 possible formats
            path = url.path[:url.path.rfind("/v1")]
            version_url = '%s://%s%s' % (url.scheme, url.netloc, path)
        else:
            # NOTE(hongbin): probably, it is one of the next cases:
            #  * https://container.example.com/
            #  * https://example.com/container
            # leave as is without cropping.
            version_url = endpoint

        return self._list(version_url, "versions")

    def get_current(self):
        for version in self.list():
            if version.status == "CURRENT":
                return version
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from osc_lib.command import command
from osc_lib import utils
from oslo_log import log as logging

from zunclient.i18n import _


def _quota_columns(quota):
    return quota._info.keys()


def _get_client(obj, parsed_args):
    obj.log.debug("take_action(%s)" % parsed_args)
    return obj.app.client_manager.container


class UpdateQuota(command.ShowOne):
    """Update the quotas of the project"""

    log = logging.getLogger(__name__ + ".UpdateQuota")

    def get_parser(self, prog_name):
        parser = super(UpdateQuota, self).get_parser(prog_name)
        parser.add_argument(
            '--containers',
            metavar='<containers>',
            help='The number of containers allowed per project')
        parser.add_argument(
            '--memory',
            metavar='<memory>',
            help='The number of megabytes of container RAM '
                 'allowed per project')
        parser.add_argument(
            '--cpu',
            metavar='<cpu>',
            help='The number of container cores or vCPUs '
                 'allowed per project')
        parser.add_argument(
            '--disk',
            metavar='<disk>',
            help='The number of gigabytes of container Disk '
                 'allowed per project')
        parser.add_argument(
            'project_id',
            metavar='<project_id>',
            help='The UUID of project in a multi-project cloud')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        opts = {}
        opts['containers'] = parsed_args.containers
        opts['memory'] = parsed_args.memory
        opts['cpu'] = parsed_args.cpu
        opts['disk'] = parsed_args.disk
        quota = client.quotas.update(parsed_args.project_id, **opts)
        columns = _quota_columns(quota)
        return columns, utils.get_item_properties(quota, columns)


class GetQuota(command.ShowOne):
    """Get quota of the project"""

    log = logging.getLogger(__name__ + '.GetQuota')

    def get_parser(self, prog_name):
        parser = super(GetQuota, self).get_parser(prog_name)
        parser.add_argument(
            '--usages',
            action='store_true',
            help='Whether show quota usage statistic or not')
        parser.add_argument(
            'project_id',
            metavar='<project_id>',
            help='The UUID of project in a multi-project cloud')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        quota = client.quotas.get(
            parsed_args.project_id,
            usages=parsed_args.usages)
        columns = _quota_columns(quota)
        return columns, utils.get_item_properties(quota, columns)


class GetDefaultQuota(command.ShowOne):
    """Get default quota of the project"""

    log = logging.getLogger(__name__ + '.GetDefaultQuota')

    def get_parser(self, prog_name):
        parser = super(GetDefaultQuota, self).get_parser(prog_name)
        parser.add_argument(
            'project_id',
            metavar='<project_id>',
            help='The UUID of project in a multi-project cloud')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        default_quota = client.quotas.defaults(parsed_args.project_id)
        columns = _quota_columns(default_quota)
        return columns, utils.get_item_properties(
            default_quota, columns)


class DeleteQuota(command.Command):
    """Delete quota of the project"""

    log = logging.getLogger(__name__ + '.DeleteQuota')

    def get_parser(self, prog_name):
        parser = super(DeleteQuota, self).get_parser(prog_name)
        parser.add_argument(
            'project_id',
            metavar='<project_id>',
            help='The UUID of project in a multi-project cloud')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        try:
            client.quotas.delete(parsed_args.project_id)
            print(_('Request to delete quotas has been accepted.'))
        except Exception as e:
            print("Delete for quotas failed: %(e)s" % {'e': e})



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import cliutils as utils
from zunclient.common import utils as zun_utils


def _show_image(image):
    utils.print_dict(image._info)


@utils.arg('image',
           metavar='<image>',
           help='Name of the image')
@utils.arg('host',
           metavar='<host>',
           help='Name or UUID of the host')
def do_pull(cs, args):
    """Pull an image into a host."""
    opts = {}
    opts['repo'] = args.image
    opts['host'] = args.host
    _show_image(cs.images.create(**opts))


@utils.arg('--marker',
           metavar='<marker>',
           default=None,
           help='The last image UUID of the previous page; '
                'displays list of images after "marker".')
@utils.arg('--limit',
           metavar='<limit>',
           type=int,
           help='Maximum number of images to return')
@utils.arg('--sort-key',
           metavar='<sort-key>',
           help='Column to sort results by')
@utils.arg('--sort-dir',
           metavar='<sort-dir>',
           choices=['desc', 'asc'],
           help='Direction to sort. "asc" or "desc".')
def do_image_list(cs, args):
    """Print a list of available images."""
    opts = {}
    opts['marker'] = args.marker
    opts['limit'] = args.limit
    opts['sort_key'] = args.sort_key
    opts['sort_dir'] = args.sort_dir
    opts = zun_utils.remove_null_parms(**opts)
    images = cs.images.list(**opts)
    columns = ('uuid', 'image_id', 'repo', 'tag', 'size')
    utils.print_list(images, columns,
                     {'versions': zun_utils.print_list_field('versions')},
                     sortby_index=None)


@utils.arg('id', metavar='<uuid>', help='UUID of image to describe.')
def do_image_show(cs, args):
    """Describe a specific image."""
    image = cs.images.get(args.id)
    _show_image(image)


@utils.arg('id',
           metavar='<uuid>',
           help='UUID of image to delete')
def do_image_delete(cs, args):
    """Delete a specified image."""
    opts = {}
    opts['image_id'] = args.id
    cs.images.delete(**opts)


@utils.arg('image',
           metavar='<image>',
           help='Name of the image')
@utils.arg('--image_driver',
           metavar='<image-driver>',
           choices=['glance', 'docker'],
           default='docker',
           help='Name of the image driver (glance, docker)')
@utils.arg('--exact-match',
           default=False,
           action='store_true',
           help='exact match image name')
def do_image_search(cs, args):
    """Print list of available images from repository based on user query."""
    opts = {}
    opts['image_driver'] = args.image_driver
    opts['exact_match'] = args.exact_match
    images = cs.images.search_image(args.image, **opts)
    columns = ('ID', 'Name', 'Tags', 'Status', 'Size', 'Metadata')
    utils.print_list(images, columns,
                     {'versions': zun_utils.print_list_field('versions')},
                     sortby_index=None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from osc_lib.command import command
from osc_lib import utils
from oslo_log import log as logging

from zunclient.common import utils as zun_utils
from zunclient import exceptions as exc
from zunclient.i18n import _


def _get_client(obj, parsed_args):
    obj.log.debug("take_action(%s)" % parsed_args)
    return obj.app.client_manager.container


class CreateRegistry(command.ShowOne):
    """Create a registry"""

    log = logging.getLogger(__name__ + ".CreateRegistry")

    def get_parser(self, prog_name):
        parser = super(CreateRegistry, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help='The name of the registry.')
        parser.add_argument(
            '--username',
            metavar='<username>',
            help='The username to login to the registry.')
        parser.add_argument(
            '--password',
            metavar='<password>',
            help='The password to login to the registry.')
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            required=True,
            help='The domain of the registry.')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        opts = {}
        opts['name'] = parsed_args.name
        opts['domain'] = parsed_args.domain
        opts['username'] = parsed_args.username
        opts['password'] = parsed_args.password
        opts = zun_utils.remove_null_parms(**opts)
        registry = client.registries.create(**opts)
        return zip(*sorted(registry._info['registry'].items()))


class ShowRegistry(command.ShowOne):
    """Show a registry"""

    log = logging.getLogger(__name__ + ".ShowRegistry")

    def get_parser(self, prog_name):
        parser = super(ShowRegistry, self).get_parser(prog_name)
        parser.add_argument(
            'registry',
            metavar='<registry>',
            help='ID or name of the registry to show.')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        opts = {}
        opts['id'] = parsed_args.registry
        opts = zun_utils.remove_null_parms(**opts)
        registry = client.registries.get(**opts)

        return zip(*sorted(registry._info['registry'].items()))


class ListRegistry(command.Lister):
    """List available registries"""

    log = logging.getLogger(__name__ + ".ListRegistrys")

    def get_parser(self, prog_name):
        parser = super(ListRegistry, self).get_parser(prog_name)
        parser.add_argument(
            '--all-projects',
            action="store_true",
            default=False,
            help='List registries in all projects')
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help='The last registry UUID of the previous page; '
                 'displays list of registries after "marker".')
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help='Maximum number of registries to return')
        parser.add_argument(
            '--sort-key',
            metavar='<sort-key>',
            help='Column to sort results by')
        parser.add_argument(
            '--sort-dir',
            metavar='<sort-dir>',
            choices=['desc', 'asc'],
            help='Direction to sort. "asc" or "desc".')
        parser.add_argument(
            '--name',
            metavar='<name>',
            help='List registries according to their name.')
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            help='List registries according to their domain.')
        parser.add_argument(
            '--project-id',
            metavar='<project-id>',
            help='List registries according to their project_id')
        parser.add_argument(
            '--user-id',
            metavar='<user-id>',
            help='List registries according to their user_id')
        parser.add_argument(
            '--username',
            metavar='<username>',
            help='List registries according to their username')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        opts = {}
        opts['all_projects'] = parsed_args.all_projects
        opts['marker'] = parsed_args.marker
        opts['limit'] = parsed_args.limit
        opts['sort_key'] = parsed_args.sort_key
        opts['sort_dir'] = parsed_args.sort_dir
        opts['domain'] = parsed_args.domain
        opts['name'] = parsed_args.name
        opts['project_id'] = parsed_args.project_id
        opts['user_id'] = parsed_args.user_id
        opts['username'] = parsed_args.username
        opts = zun_utils.remove_null_parms(**opts)
        registries = client.registries.list(**opts)
        columns = ('uuid', 'name', 'domain', 'username', 'password')
        return (columns, (utils.get_item_properties(registry, columns)
                          for registry in registries))


class DeleteRegistry(command.Command):
    """Delete specified registry(s)"""

    log = logging.getLogger(__name__ + ".Deleteregistry")

    def get_parser(self, prog_name):
        parser = super(DeleteRegistry, self).get_parser(prog_name)
        parser.add_argument(
            'registry',
            metavar='<registry>',
            nargs='+',
            help='ID or name of the registry(s) to delete.')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        registries = parsed_args.registry
        for registry in registries:
            opts = {}
            opts['id'] = registry
            opts = zun_utils.remove_null_parms(**opts)
            try:
                client.registries.delete(**opts)
                print(_('Request to delete registry %s has been accepted.')
                      % registry)
            except Exception as e:
                print("Delete for registry %(registry)s failed: %(e)s" %
                      {'registry': registry, 'e': e})


class UpdateRegistry(command.ShowOne):
    """Update one or more attributes of the registry"""
    log = logging.getLogger(__name__ + ".UpdateRegistry")

    def get_parser(self, prog_name):
        parser = super(UpdateRegistry, self).get_parser(prog_name)
        parser.add_argument(
            'registry',
            metavar='<registry>',
            help="ID or name of the registry to update.")
        parser.add_argument(
            '--username',
            metavar='<username>',
            help='The new username of registry to update.')
        parser.add_argument(
            '--password',
            metavar='<password>',
            help='The new password of registry to update.')
        parser.add_argument(
            '--name',
            metavar='<name>',
            help='The new name of registry to update.')
        parser.add_argument(
            '--domain',
            metavar='<domain>',
            help='The new domain of registry to update.')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        registry = parsed_args.registry
        opts = {}
        opts['username'] = parsed_args.username
        opts['password'] = parsed_args.password
        opts['domain'] = parsed_args.domain
        opts['name'] = parsed_args.name
        opts = zun_utils.remove_null_parms(**opts)
        if not opts:
            raise exc.CommandError("You must update at least one property")
        registry = client.registries.update(registry, **opts)
        return zip(*sorted(registry._info['registry'].items()))



# Copyright (c) 2015 Thales Services SAS
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import testtools

from zunclient.v1 import client


class ClientTest(testtools.TestCase):

    @mock.patch('zunclient.common.httpclient.SessionClient')
    @mock.patch('keystoneauth1.session.Session')
    def test_init_with_session(self, mock_session, http_client):
        session = mock.Mock()
        client.Client(session=session)
        mock_session.assert_not_called()
        http_client.assert_called_once_with(
            interface='public',
            region_name=None,
            service_name=None,
            service_type='container',
            session=session,
            api_version=None)

    @mock.patch('zunclient.common.httpclient.SessionClient')
    @mock.patch('keystoneauth1.token_endpoint.Token')
    @mock.patch('keystoneauth1.session.Session')
    def test_init_with_token_and_url(
            self, mock_session, mock_token, http_client):
        mock_auth_plugin = mock.Mock()
        mock_token.return_value = mock_auth_plugin
        session = mock.Mock()
        mock_session.return_value = session
        client.Client(auth_token='mytoken',
                      endpoint_override='http://myurl/')
        mock_session.assert_called_once_with(
            auth=mock_auth_plugin, cert=None, verify=True)
        http_client.assert_called_once_with(
            endpoint_override='http://myurl/',
            interface='public',
            region_name=None,
            service_name=None,
            service_type='container',
            session=session,
            api_version=None)

    @mock.patch('zunclient.common.httpclient.SessionClient')
    @mock.patch('keystoneauth1.loading.get_plugin_loader')
    @mock.patch('keystoneauth1.session.Session')
    def test_init_with_token(
            self, mock_session, mock_loader, http_client):
        mock_plugin = mock.Mock()
        mock_loader.return_value = mock_plugin
        client.Client(auth_token='mytoken', auth_url='authurl')
        mock_loader.assert_called_once_with('token')
        mock_plugin.load_from_options.assert_called_once_with(
            auth_url='authurl',
            project_id=None,
            project_name=None,
            project_domain_id=None,
            project_domain_name=None,
            user_domain_id=None,
            user_domain_name=None,
            token='mytoken')
        http_client.assert_called_once_with(
            interface='public',
            region_name=None,
            service_name=None,
            service_type='container',
            session=mock.ANY,
            api_version=None)

    @mock.patch('zunclient.common.httpclient.SessionClient')
    @mock.patch('keystoneauth1.loading.get_plugin_loader')
    @mock.patch('keystoneauth1.session.Session')
    def test_init_with_user(
            self, mock_session, mock_loader, http_client):
        mock_plugin = mock.Mock()
        mock_loader.return_value = mock_plugin
        client.Client(username='myuser', auth_url='authurl')
        mock_loader.assert_called_once_with('password')
        mock_plugin.load_from_options.assert_called_once_with(
            auth_url='authurl',
            username='myuser',
            password=None,
            project_domain_id=None,
            project_domain_name=None,
            user_domain_id=None,
            user_domain_name=None,
            project_id=None,
            project_name=None)
        http_client.assert_called_once_with(
            interface='public',
            region_name=None,
            service_name=None,
            service_type='container',
            session=mock.ANY,
            api_version=None)

    @mock.patch('zunclient.common.httpclient.SessionClient')
    @mock.patch('keystoneauth1.loading.get_plugin_loader')
    @mock.patch('keystoneauth1.session.Session')
    def test_init_unauthorized(
            self, mock_session, mock_loader, http_client):
        mock_plugin = mock.Mock()
        mock_loader.return_value = mock_plugin
        mock_session_obj = mock.Mock()
        mock_session.return_value = mock_session_obj
        mock_session_obj.get_endpoint.side_effect = Exception()
        self.assertRaises(
            RuntimeError,
            client.Client, username='myuser', auth_url='authurl')
        mock_loader.assert_called_once_with('password')
        mock_plugin.load_from_options.assert_called_once_with(
            auth_url='authurl',
            username='myuser',
            password=None,
            project_domain_id=None,
            project_domain_name=None,
            user_domain_id=None,
            user_domain_name=None,
            project_id=None,
            project_name=None)
        http_client.assert_not_called()

    @mock.patch('zunclient.common.httpclient.SessionClient')
    @mock.patch('keystoneauth1.session.Session')
    def test_init_with_endpoint_override(self, mock_session, http_client):
        session = mock.Mock()
        client.Client(session=session, endpoint_override='zunurl')
        mock_session.assert_not_called()
        http_client.assert_called_once_with(
            interface='public',
            region_name=None,
            service_name=None,
            service_type='container',
            session=session,
            endpoint_override='zunurl',
            api_version=None)

    @mock.patch('zunclient.common.httpclient.SessionClient')
    @mock.patch('keystoneauth1.session.Session')
    def test_init_with_zun_url_and_endpoint_override(self, mock_session,
                                                     http_client):
        session = mock.Mock()
        client.Client(session=session, zun_url='zunurl',
                      endpoint_override='zunurl')
        mock_session.assert_not_called()
        http_client.assert_called_once_with(
            interface='public',
            region_name=None,
            service_name=None,
            service_type='container',
            session=session,
            endpoint_override='zunurl',
            api_version=None)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import cliutils as utils


@utils.arg(
    '--containers',
    metavar='<containers>',
    type=int,
    help='The number of containers allowed per project')
@utils.arg(
    '--cpu',
    metavar='<cpu>',
    type=int,
    help='The number of container cores or vCPUs allowed per project')
@utils.arg(
    '--memory',
    metavar='<memory>',
    type=int,
    help='The number of megabytes of container RAM allowed per project')
@utils.arg(
    '--disk',
    metavar='<disk>',
    type=int,
    help='The number of gigabytes of container Disk allowed per project')
@utils.arg(
    'project_id',
    metavar='<project_id>',
    help='The UUID of project in a multi-project cloud')
def do_quota_update(cs, args):
    """Print an updated quotas for a project"""
    utils.print_dict(cs.quotas.update(args.project_id,
                                      containers=args.containers,
                                      memory=args.memory,
                                      cpu=args.cpu,
                                      disk=args.disk)._info)


@utils.arg(
    '--usages',
    default=False,
    action='store_true',
    help='Whether show quota usage statistic or not')
@utils.arg(
    'project_id',
    metavar='<project_id>',
    help='The UUID of project in a multi-project cloud')
def do_quota_get(cs, args):
    """Print a quotas for a project with usages (optional)"""
    if args.usages:
        utils.print_dict(
            cs.quotas.get(args.project_id, usages=args.usages)._info,
            value_fields=('limit', 'in_use'))
    else:
        utils.print_dict(
            cs.quotas.get(args.project_id, usages=args.usages)._info)


@utils.arg(
    'project_id',
    metavar='<project_id>',
    help='The UUID of project in a multi-project cloud')
def do_quota_defaults(cs, args):
    """Print a  default quotas for a project"""
    utils.print_dict(cs.quotas.defaults(args.project_id)._info)


@utils.arg(
    'project_id',
    metavar='<project_id>',
    help='The UUID of project in a multi-project cloud')
def do_quota_delete(cs, args):
    """Delete quotas for a project"""
    cs.quotas.delete(args.project_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from zunclient.common.apiclient import exceptions as apiexec
from zunclient.common import utils as zun_utils
from zunclient.common.websocketclient import exceptions
from zunclient.tests.unit.v1 import shell_test_base
from zunclient.v1 import containers_shell


def _get_container_args(**kwargs):
    default_args = {
        'auto_remove': False,
        'environment': {},
        'hints': {},
        'labels': {},
        'mounts': [],
        'nets': [],
        'command': [],
    }
    default_args.update(kwargs)
    return default_args


class ShellTest(shell_test_base.TestCommandLineArgument):

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.create')
    def test_zun_container_create_success(self, mock_create,
                                          mock_show_container):
        mock_create.return_value = 'container'
        self._test_arg_success('create x')
        mock_show_container.assert_called_once_with('container')

    @mock.patch('zunclient.common.utils.list_containers')
    @mock.patch('zunclient.v1.containers.ContainerManager.list')
    def test_zun_container_list_success(self, mock_list, mock_list_containers):
        mock_list.return_value = ['container']
        self._test_arg_success('list')
        mock_list_containers.assert_called_once_with(['container'])

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.get')
    def test_zun_container_show_success(self, mock_get, mock_show_container):
        mock_get.return_value = 'container'
        self._test_arg_success('show x')
        mock_show_container.assert_called_once_with('container')

    @mock.patch('zunclient.v1.containers.ContainerManager.list')
    def test_zun_container_command_failure(self, mock_list):
        self._test_arg_failure('list --wrong',
                               self._unrecognized_arg_error)
        self.assertFalse(mock_list.called)

    @mock.patch('zunclient.common.cliutils.print_dict')
    def test_show_container(self, mock_print_dict):
        fake_container = mock.MagicMock()
        fake_container._info = {}
        fake_container.addresses = {'private': [{'addr': '10.0.0.1'}]}
        containers_shell._show_container(fake_container)
        mock_print_dict.assert_called_once_with({'networks': 'private',
                                                 'addresses': '10.0.0.1'})

    @mock.patch('zunclient.common.cliutils.print_list')
    def test_list_container(self, mock_print_list):
        fake_container = mock.MagicMock()
        fake_container._info = {}
        fake_container.addresses = {'private': [{'addr': '10.0.0.1'}]}
        zun_utils.list_containers([fake_container])
        self.assertTrue(mock_print_list.called)
        self.assertEqual(fake_container.addresses, '10.0.0.1')

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.create')
    def test_zun_container_create_success_with_pull_policy(
            self, mock_create, mock_show_container):
        mock_create.return_value = 'container-never'
        self._test_arg_success(
            'create --image-pull-policy never x')
        mock_show_container.assert_called_with('container-never')
        mock_create.assert_called_with(
            **_get_container_args(image='x', image_pull_policy='never'))

        mock_create.return_value = 'container-always'
        self._test_arg_success(
            'create --image-pull-policy always x')
        mock_show_container.assert_called_with('container-always')
        mock_create.assert_called_with(
            **_get_container_args(image='x', image_pull_policy='always'))

        mock_create.return_value = 'container-ifnotpresent'
        self._test_arg_success(
            'create --image-pull-policy ifnotpresent x')
        mock_show_container.assert_called_with('container-ifnotpresent')
        mock_create.assert_called_with(
            **_get_container_args(image='x', image_pull_policy='ifnotpresent'))

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.create')
    def test_zun_container_create_success_without_pull_policy(
            self, mock_create, mock_show_container):
        mock_create.return_value = 'container'
        self._test_arg_success('create x')
        mock_show_container.assert_called_once_with('container')
        mock_create.assert_called_with(**_get_container_args(image='x'))

    @mock.patch('zunclient.v1.containers.ContainerManager.create')
    def test_zun_container_create_failure_with_wrong_pull_policy(
            self, mock_create):
        self._test_arg_failure(
            'create --image-pull-policy wrong x ',
            self._invalid_choice_error)
        self.assertFalse(mock_create.called)
        mock_create.assert_not_called()

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_success_with_pull_policy(
            self, mock_run, mock_show_container):
        mock_run.return_value = 'container-never'
        self._test_arg_success(
            'run --image-pull-policy never x')
        mock_show_container.assert_called_with('container-never')
        mock_run.assert_called_with(
            **_get_container_args(image='x', image_pull_policy='never'))

        mock_run.return_value = 'container-always'
        self._test_arg_success(
            'run --image-pull-policy always x ')
        mock_show_container.assert_called_with('container-always')
        mock_run.assert_called_with(
            **_get_container_args(image='x', image_pull_policy='always'))

        mock_run.return_value = 'container-ifnotpresent'
        self._test_arg_success(
            'run --image-pull-policy ifnotpresent x')
        mock_show_container.assert_called_with('container-ifnotpresent')
        mock_run.assert_called_with(
            **_get_container_args(image='x', image_pull_policy='ifnotpresent'))

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_success_without_pull_policy(
            self, mock_run, mock_show_container):
        mock_run.return_value = 'container'
        self._test_arg_success('run x')
        mock_show_container.assert_called_once_with('container')
        mock_run.assert_called_with(**_get_container_args(image='x'))

    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_failure_with_wrong_pull_policy(
            self, mock_run):
        self._test_arg_failure(
            'run --image-pull-policy wrong x',
            self._invalid_choice_error)
        self.assertFalse(mock_run.called)
        mock_run.assert_not_called()

    @mock.patch('zunclient.v1.containers.ContainerManager.get')
    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_interactive(self, mock_run,
                                           mock_show_container,
                                           mock_get_container):
        fake_container = mock.MagicMock()
        fake_container.uuid = 'fake_uuid'
        mock_run.return_value = fake_container
        fake_container.status = 'Error'
        mock_get_container.return_value = fake_container
        self.assertRaises(exceptions.ContainerStateError,
                          self.shell,
                          'run -i x ')

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_success_with_runtime(
            self, mock_run, mock_show_container):
        mock_run.return_value = 'container'
        self._test_arg_success('run --runtime runc x')
        mock_show_container.assert_called_once_with('container')
        mock_run.assert_called_with(
            **_get_container_args(image='x', runtime='runc'))

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_success_with_availability_zone(
            self, mock_run, mock_show_container):
        mock_run.return_value = 'container'
        self._test_arg_success('run --availability-zone nova x')
        mock_show_container.assert_called_once_with('container')

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_with_mount(
            self, mock_run, mock_show_container):
        mock_run.return_value = 'container'
        self._test_arg_success(
            'run --mount source=s,destination=d x')
        mock_show_container.assert_called_once_with('container')
        mounts = [{'source': 's', 'destination': 'd'}]
        mock_run.assert_called_with(
            **_get_container_args(image='x', mounts=mounts))

    def test_zun_container_run_with_mount_invalid_format(self):
        self.assertRaisesRegex(
            apiexec.CommandError, 'Invalid mounts argument',
            self.shell,
            'run --mount source,destination=d x')

    def test_zun_container_run_with_mount_missed_key(self):
        self.assertRaisesRegex(
            apiexec.CommandError, 'Invalid mounts argument',
            self.shell,
            'run --mount source=s x')

    def test_zun_container_run_with_mount_duplicated_key(self):
        self.assertRaisesRegex(
            apiexec.CommandError, 'Invalid mounts argument',
            self.shell,
            'run --mount source=s,source=s,destination=d x')

    def test_zun_container_run_with_mount_invalid_key(self):
        self.assertRaisesRegex(
            apiexec.CommandError, 'Invalid mounts argument',
            self.shell,
            'run --mount invalid=s,destination=d x')

    @mock.patch('zunclient.v1.containers_shell._show_container')
    @mock.patch('zunclient.v1.containers.ContainerManager.run')
    def test_zun_container_run_success_with_hostname(
        self, mock_run, mock_show):
        mock_run.return_value = 'container'
        self._test_arg_success('run --hostname testhost x')
        mock_show.assert_called_once_with('container')
        mock_run.assert_called_with(
            **_get_container_args(image='x', hostname='testhost'))



# Copyright (c) 2015 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import warnings

from oslo_utils import importutils

from zunclient import api_versions

osprofiler_profiler = importutils.try_import("osprofiler.profiler")


def _get_client_class_and_version(version):
    if not isinstance(version, api_versions.APIVersion):
        version = api_versions.get_api_version(version)
    else:
        api_versions.check_major_version(version)
    return version, importutils.import_class(
        'zunclient.v%s.client.Client' % version.ver_major)


def _check_arguments(kwargs, release, deprecated_name, right_name=None):
    """Process deprecation of arguments.

    Check presence of deprecated argument in kwargs, prints proper warning
    message, renames key to right one it needed.
    """
    if deprecated_name in kwargs:
        if right_name:
            if right_name in kwargs:
                msg = ('The %(old)s argument is deprecated in %(release)s'
                       'and its use may result in errors in future releases.'
                       'As %(new)s is provided, the %(old)s argument will '
                       'be ignored.') % {'old': deprecated_name,
                                         'release': release,
                                         'new': right_name}
                kwargs.pop(deprecated_name)
            else:
                msg = ('The %(old)s argument is deprecated in %(release)s '
                       'and its use may result in errors in future releases. '
                       'Use %(new)s instead.') % {'old': deprecated_name,
                                                  'release': release,
                                                  'new': right_name}
                kwargs[right_name] = kwargs.pop(deprecated_name)
        else:
            msg = ('The %(old)s argument is deprecated in %(release)s '
                   'and its use may result in errors in future '
                   'releases') % {'old': deprecated_name,
                                  'release': release}
            # NOTE(kiennt): just ignore it
            kwargs.pop(deprecated_name)
        warnings.warn(msg)


def Client(version='1', username=None, auth_url=None, **kwargs):
    """Initialize client objects based on given version"""
    _check_arguments(kwargs, 'Queens', 'api_key', right_name='password')
    # NOTE: OpenStack projects use 2 vars with one meaning: `endpoint_type`
    #       and `interface`. `endpoint_type` is an old name which was used by
    #       most OpenStack clients. Later it was replaced by `interface` in
    #       keystone and later some other clients switched to new var name too.
    _check_arguments(kwargs, 'Queens', 'endpoint_type',
                     right_name='interface')
    _check_arguments(kwargs, 'Queens', 'zun_url',
                     right_name='endpoint_override')
    _check_arguments(kwargs, 'Queens', 'tenant_name',
                     right_name='project_name')
    _check_arguments(kwargs, 'Queens', 'tenant_id', right_name='project_id')

    profile = kwargs.pop('profile', None)
    if osprofiler_profiler and profile:
        # Initialize the root of the future trace: the created trace ID
        # will be used as the very first parent to which all related
        # traces will be bound to. The given HMAC key must correspond to
        # the one set in zun-api zun.conf, otherwise the latter
        # will fail to check the request signature and will skip
        # initialization of osprofiler on the server side.
        osprofiler_profiler.init(profile)

    api_version, client_class = _get_client_class_and_version(version)
    if api_version.is_latest():
        c = client_class(api_version=api_versions.APIVersion("1.1"),
                         auth_url=auth_url,
                         username=username,
                         **kwargs)
        api_version = api_versions.discover_version(c, api_version)

    return client_class(api_version=api_version,
                        auth_url=auth_url,
                        username=username,
                        **kwargs)



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from zunclient.v1 import actions_shell
from zunclient.v1 import availability_zones_shell
from zunclient.v1 import capsules_shell
from zunclient.v1 import containers_shell
from zunclient.v1 import hosts_shell
from zunclient.v1 import images_shell
from zunclient.v1 import quota_classes_shell
from zunclient.v1 import quotas_shell
from zunclient.v1 import registries_shell
from zunclient.v1 import services_shell
from zunclient.v1 import versions_shell

COMMAND_MODULES = [
    availability_zones_shell,
    containers_shell,
    images_shell,
    services_shell,
    hosts_shell,
    versions_shell,
    capsules_shell,
    actions_shell,
    quotas_shell,
    quota_classes_shell,
    registries_shell,
]



# Copyright (c) 2015 IBM Corp.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import testtools

from zunclient import api_versions
from zunclient import client
from zunclient import exceptions


class ClientTest(testtools.TestCase):

    @mock.patch('zunclient.api_versions.discover_version',
                return_value=api_versions.APIVersion('1.1'))
    @mock.patch('zunclient.v1.client.Client')
    def test_no_version_argument(self, mock_zun_client_v1,
                                 mock_discover_version):
        client.Client(auth_url='http://example/identity',
                      username='admin')
        mock_zun_client_v1.assert_called_with(
            api_version=api_versions.APIVersion('1.1'),
            auth_url='http://example/identity',
            username='admin')

    @mock.patch('zunclient.api_versions.discover_version',
                return_value=api_versions.APIVersion('1.1'))
    @mock.patch('zunclient.v1.client.Client')
    def test_valid_version_argument(self, mock_zun_client_v1,
                                    mock_discover_version):
        client.Client(version='1',
                      auth_url='http://example/identity',
                      username='admin')
        mock_zun_client_v1.assert_called_with(
            api_version=api_versions.APIVersion('1.1'),
            auth_url='http://example/identity',
            username='admin')

    def test_invalid_version_argument(self):
        self.assertRaises(
            exceptions.UnsupportedVersion,
            client.Client, version='2')



# Copyright 2014
# The Cloudscaling Group, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class ContainerWebSocketException(Exception):
    """base for all ContainerWebSocket interactive generated exceptions"""
    def __init__(self, wrapped=None, message=None):
        self.wrapped = wrapped
        if message:
            self.message = message
        if wrapped:
            formatted_string = "%s:%s" % (self.message, str(self.wrapped))
        else:
            formatted_string = "%s" % self.message
        super(ContainerWebSocketException, self).__init__(formatted_string)


class UserExit(ContainerWebSocketException):
    message = "User requested disconnect the container"


class Disconnected(ContainerWebSocketException):
    message = "Remote host closed connection"


class ConnectionFailed(ContainerWebSocketException):
    message = "Failed to connect to remote host"


class InvalidWebSocketLink(ContainerWebSocketException):
    message = "Invalid websocket link when attach container"


class ContainerFailtoStart(ContainerWebSocketException):
    message = "Container fail to start"


class ContainerStateError(ContainerWebSocketException):
    message = "Container state is error, can not attach container"



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from zunclient import api_versions
from zunclient.osc import plugin
from zunclient.tests.unit import base


class TestContainerPlugin(base.TestCase):

    @mock.patch("zunclient.api_versions.get_api_version")
    @mock.patch("zunclient.v1.client.Client")
    def test_make_client(self, p_client, mock_get_api_version):

        instance = mock.Mock()
        instance._api_version = {"container": '1'}
        instance._region_name = 'zun_region'
        instance.session = 'zun_session'
        mock_get_api_version.return_value = api_versions.APIVersion('1.2')

        plugin.make_client(instance)
        p_client.assert_called_with(region_name='zun_region',
                                    session='zun_session',
                                    service_type='container',
                                    api_version=api_versions.APIVersion('1.2'))



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zunclient.common import base


class AvailabilityZone(base.Resource):
    def __repr__(self):
        return "<AvailabilityZone %s>" % self._info


class AvailabilityZoneManager(base.Manager):
    resource_class = AvailabilityZone

    @staticmethod
    def _path():
        return '/v1/availability_zones'

    def list(self, **kwargs):
        return self._list(self._path(), "availability_zones", qparams=kwargs)



# Copyright 2012 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import datetime
import io
import os
import sys

import fixtures
import testtools

from zunclient import api_versions
from zunclient.common import httpclient as http
from zunclient import shell

FAKE_ENV = {'OS_USERNAME': 'username',
            'OS_PASSWORD': 'password',
            'OS_PROJECT_NAME': 'project_name',
            'OS_AUTH_URL': 'http://no.where/v2.0'}


class BaseTestCase(testtools.TestCase):

    def setUp(self):
        super(BaseTestCase, self).setUp()
        self.useFixture(fixtures.FakeLogger())


class FakeAPI(object):
    def __init__(self, responses, endpoint=None):
        self.responses = responses
        self.calls = []
        self.api_version = api_versions.APIVersion(
            api_versions.MAX_API_VERSION)
        self.endpoint = endpoint

    def _request(self, method, url, headers=None, body=None):
        call = (method, url, headers or {}, body)
        self.calls.append(call)
        return self.responses[url][method]

    def raw_request(self, *args, **kwargs):
        response = self._request(*args, **kwargs)
        body_iter = http.ResponseBodyIterator(io.StringIO(response[1]))
        return FakeResponse(response[0]), body_iter

    def json_request(self, *args, **kwargs):
        response = self._request(*args, **kwargs)
        return FakeResponse(response[0]), response[1]

    def get_endpoint(self, *args, **kwargs):
        return self.endpoint or '/v1'


class FakeConnection(object):
    def __init__(self, response=None):
        self._response = response
        self._last_request = None

    def request(self, method, conn_url, **kwargs):
        self._last_request = (method, conn_url, kwargs)

    def setresponse(self, response):
        self._response = response

    def getresponse(self):
        return self._response


class FakeResponse(object):
    def __init__(self, headers, body=None, version=None, status=None,
                 reason=None):
        """Fake object to help testing.

        :param headers: dict representing HTTP response headers
        :param body: file-like object
        """
        self.headers = headers
        self.body = body
        self.version = version
        self.status = status
        self.reason = reason

    def getheaders(self):
        return copy.deepcopy(self.headers).items()

    def getheader(self, key, default):
        return self.headers.get(key, default)

    def read(self, amt):
        return self.body.read(amt)


class FakeServiceCatalog(object):
    def url_for(self, endpoint_type, service_type, attr=None,
                filter_value=None):
        if attr == 'region' and filter_value:
            return 'http://regionhost:6385/v1/f14b41234'
        else:
            return 'http://localhost:6385/v1/f14b41234'


class FakeKeystone(object):
    service_catalog = FakeServiceCatalog()
    timestamp = datetime.datetime.utcnow() + datetime.timedelta(days=5)

    def __init__(self, auth_token):
        self.auth_token = auth_token
        self.auth_ref = {
            'token': {'expires': FakeKeystone.timestamp.strftime(
                      '%Y-%m-%dT%H:%M:%S.%f'),
                      'id': 'd1a541311782870742235'}
        }


class TestCase(testtools.TestCase):
    TEST_REQUEST_BASE = {
        'verify': True,
    }

    def setUp(self):
        super(TestCase, self).setUp()
        if (os.environ.get('OS_STDOUT_CAPTURE') == 'True' or
                os.environ.get('OS_STDOUT_CAPTURE') == '1'):
            stdout = self.useFixture(fixtures.StringStream('stdout')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stdout', stdout))
        if (os.environ.get('OS_STDERR_CAPTURE') == 'True' or
                os.environ.get('OS_STDERR_CAPTURE') == '1'):
            stderr = self.useFixture(fixtures.StringStream('stderr')).stream
            self.useFixture(fixtures.MonkeyPatch('sys.stderr', stderr))

    def make_env(self, exclude=None, fake_env=FAKE_ENV):
        env = dict((k, v) for k, v in fake_env.items() if k != exclude)
        self.useFixture(fixtures.MonkeyPatch('os.environ', env))

    def shell(self, argstr, exitcodes=(0,)):
        orig = sys.stdout
        orig_stderr = sys.stderr
        try:
            sys.stdout = io.StringIO()
            sys.stderr = io.StringIO()
            _shell = shell.OpenStackZunShell()
            _shell.main(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertIn(exc_value.code, exitcodes)
        finally:
            stdout = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig
            stderr = sys.stderr.getvalue()
            sys.stderr.close()
            sys.stderr = orig_stderr
        return (stdout, stderr)


class FakeSessionResponse(object):

    def __init__(self, headers, content=None, status_code=None):
        self.headers = headers
        self.content = content
        self.status_code = status_code


class FakeSession(object):

    def __init__(self, headers, content=None, status_code=None):
        self.headers = headers
        self.content = content
        self.status_code = status_code

    def request(self, url, method, **kwargs):
        return FakeSessionResponse(self.headers, self.content,
                                   self.status_code)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging

from osc_lib.command import command
from osc_lib import utils

from zunclient.common import utils as zun_utils


def _host_columns(host):
    return host._info.keys()


def _get_client(obj, parsed_args):
    obj.log.debug("take_action(%s)" % parsed_args)
    return obj.app.client_manager.container


class ListHost(command.Lister):
    """List available hosts"""

    log = logging.getLogger(__name__ + ".ListHost")

    def get_parser(self, prog_name):
        parser = super(ListHost, self).get_parser(prog_name)
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            default=None,
            help='The last host UUID of the previous page; '
                 'displays list of hosts after "marker".')
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help='Maximum number of hosts to return')
        parser.add_argument(
            '--sort-key',
            metavar='<sort-key>',
            help='Column to sort results by')
        parser.add_argument(
            '--sort-dir',
            metavar='<sort-dir>',
            choices=['desc', 'asc'],
            help='Direction to sort. "asc" or "desc".')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        opts = {}
        opts['marker'] = parsed_args.marker
        opts['limit'] = parsed_args.limit
        opts['sort_key'] = parsed_args.sort_key
        opts['sort_dir'] = parsed_args.sort_dir
        opts = zun_utils.remove_null_parms(**opts)
        hosts = client.hosts.list(**opts)
        columns = ('uuid', 'hostname', 'mem_total', 'cpus', 'disk_total')
        return (columns, (utils.get_item_properties(host, columns)
                          for host in hosts))


class ShowHost(command.ShowOne):
    """Show a host"""

    log = logging.getLogger(__name__ + ".ShowHost")

    def get_parser(self, prog_name):
        parser = super(ShowHost, self).get_parser(prog_name)
        parser.add_argument(
            'host',
            metavar='<host>',
            help='ID or name of the host to show.')
        return parser

    def take_action(self, parsed_args):
        client = _get_client(self, parsed_args)
        host = parsed_args.host
        host = client.hosts.get(host)
        columns = _host_columns(host)

        return columns, utils.get_item_properties(host, columns)
